"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseRepository = void 0;
const ramda_1 = __importDefault(require("ramda"));
class BaseRepository {
    // model is objection Model
    constructor(model, knexInstance) {
        this.model = model;
        this.knexInstance = knexInstance;
    }
    createMany(items, options) {
        return this.model.query(options === null || options === void 0 ? void 0 : options.trx).insert(items).returning('*');
    }
    create(data, options = {}) {
        return this.model.query(options === null || options === void 0 ? void 0 : options.trx).insert(data).returning('*');
    }
    delete(conditions, options = {}) {
        let query = this.model.query(options === null || options === void 0 ? void 0 : options.trx).delete();
        if (!ramda_1.default.isEmpty(conditions)) {
            query = query.where(conditions);
        }
        if (!ramda_1.default.isNil(options.whereIn)) {
            options.whereIn.forEach((condition) => {
                query = query.whereIn(condition.field, condition.values);
            });
        }
        if (!ramda_1.default.isNil(options.whereNotIn)) {
            options.whereNotIn.forEach((condition) => {
                query = query.whereNotIn(condition.field, condition.values);
            });
        }
        return query;
    }
    update(conditions, data, options = {}) {
        let query = this.model.query(options === null || options === void 0 ? void 0 : options.trx).patch(data).where(conditions);
        if (!ramda_1.default.isNil(options.whereIn)) {
            options.whereIn.forEach((condition) => {
                query = query.whereIn(condition.field, condition.values);
            });
        }
        if (!ramda_1.default.isNil(options.whereNotIn)) {
            options.whereNotIn.forEach((condition) => {
                query = query.whereNotIn(condition.field, condition.values);
            });
        }
        return query;
    }
    getOne(conditions, options = {}) {
        let query = this.model.query(options === null || options === void 0 ? void 0 : options.trx).findOne(conditions);
        if (!ramda_1.default.isNil(options.whereIn)) {
            options.whereIn.forEach((condition) => {
                query = query.whereIn(condition.field, condition.values);
            });
        }
        if (!ramda_1.default.isNil(options.whereNotIn)) {
            options.whereNotIn.forEach((condition) => {
                query = query.whereNotIn(condition.field, condition.values);
            });
        }
        if (options.forUpdate) {
            query = query.forUpdate();
        }
        return query;
    }
    getAll(conditions, options = {}) {
        let query = this.model.query(options === null || options === void 0 ? void 0 : options.trx).where(conditions);
        if (!ramda_1.default.isNil(options.whereIn)) {
            options.whereIn.forEach((condition) => {
                query = query.whereIn(condition.field, condition.values);
            });
        }
        if (!ramda_1.default.isNil(options.whereNotIn)) {
            options.whereNotIn.forEach((condition) => {
                query = query.whereNotIn(condition.field, condition.values);
            });
        }
        if (options.forUpdate) {
            query = query.forUpdate();
        }
        return query;
    }
    // Shortcut for Query Builder call
    qb() {
        return this.model.knex().table(this.model.tableName);
    }
    knex() {
        return this.model.knex();
    }
    getWithRelatedModels(conditions, options = { relatedModels: {} }) {
        return __awaiter(this, void 0, void 0, function* () {
            let query = this.model
                .query(options === null || options === void 0 ? void 0 : options.trx)
                .withGraphFetched(Object.assign({}, options.relatedModels))
                .where(conditions);
            if (!ramda_1.default.isNil(options.whereIn)) {
                options.whereIn.forEach((condition) => {
                    query = query.whereIn(condition.field, condition.values);
                });
            }
            if (!ramda_1.default.isNil(options.whereNotIn)) {
                options.whereNotIn.forEach((condition) => {
                    query = query.whereNotIn(condition.field, condition.values);
                });
            }
            return query;
        });
    }
    getKnexInstance() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.knexInstance;
        });
    }
}
exports.BaseRepository = BaseRepository;
