"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const database_1 = require("./helpers/database");
const user_repository_1 = require("./helpers/modules/users/user.repository");
describe('ObjectionJS Repository Testing', function () {
    let userRepo;
    before(function () {
        return __awaiter(this, void 0, void 0, function* () {
            userRepo = new user_repository_1.UserRepository(yield database_1.Database.getInstance());
        });
    });
    beforeEach(function () {
        return __awaiter(this, void 0, void 0, function* () {
            yield database_1.Database.seed();
        });
    });
    describe('Read functions testing', function () {
        it('Should get one user.', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const user = yield userRepo.getOne({ age: 25 });
                assert_1.default.equal(25, user === null || user === void 0 ? void 0 : user.age);
                assert_1.default.equal('Ahmed', user === null || user === void 0 ? void 0 : user.name);
            });
        });
        it('Should not get one user.', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const user = yield userRepo.getOne({ age: 40 });
                assert_1.default.equal(undefined, user);
            });
        });
        it('Should get all users.', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const users = yield userRepo.getAll({});
                assert_1.default.equal(2, users.length);
            });
        });
        it('Should get all users with condition.', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const users = yield userRepo.getAll({ name: 'Ali' });
                assert_1.default.equal(1, users.length);
                assert_1.default.equal('Ali', users[0].name);
            });
        });
    });
    describe('Write functions testing', function () {
        it('Should create user.', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const userToBeInserted = {
                    name: 'Akin',
                    age: 28,
                };
                yield userRepo.create(userToBeInserted);
                const user = yield userRepo.getOne(userToBeInserted);
                assert_1.default.equal(userToBeInserted.age, user === null || user === void 0 ? void 0 : user.age);
                assert_1.default.equal(userToBeInserted.name, user === null || user === void 0 ? void 0 : user.name);
            });
        });
    });
});
